/*
 * Org chart
 * Developer: Vinyl
 * Date: Thursday, February 7, 2019
 */
define(function () {

    var install = function (holderElement, context) {
        var divContainer = holderElement.childNodes[0];

        var apiKey = context.getParameter("ApiKey");
        
        var property_id = context.getParameter("property.id");
        var property_parent = context.getParameter("property.parent");
        var property_label = context.getParameter("property.label");
        var property_subtitle = context.getParameter("property.subtitle");
        var property_items = context.getParameter("property.items");

        var support_linktopage = context.getParameter("support.linktopage") == 1;
        var support_wrapping = context.getParameter("support.wrapping") == 1; // Using == because this returns strings

        function list_to_tree(list) {
            var map = {}, node, roots = [], i;
            for (i = 0; i < list.length; i += 1) {
                map[list[i][property_id]] = i; // initialize the map
                list[i].children = []; // initialize the children
            }
            for (i = 0; i < list.length; i += 1) {
                node = list[i];
                if (node[property_parent] !== null) {
                    // if you have dangling branches check that map[node.parentId] exists
                    list[map[node[property_parent]]].children.push(node);
                } else {
                    roots.push(node);
                }
            }
            return roots;
        }
        
        var callback = function(result) {
            var rows = property_items ? result[property_items] : result;

            var resultParsed = list_to_tree(rows);
            
            if (!resultParsed || resultParsed.length !== 1) {
                console.log("Org chart results do not return a single root node", resultParsed);
            }
            
            var initCompleted = function($chart) {
                $divContainer.css('overflow', 'visible');

                // Their hover is ugly
                $divContainer.find(".node").off("mouseenter mouseleave");
            };

            var $divContainer = $(divContainer);
            var options = {
                "data": resultParsed[0],
                "nodeId": property_id,
                "nodeTitle": property_label,
                "pan": true,
                "zoom": true,
                "parentNodeSymbol": null,
                
                "initCompleted": initCompleted
            };
            
            if (property_subtitle) {
                options["nodeContent"] = property_subtitle;
            }
            
            if (support_linktopage) {
                var activate = function(data) {
                    var fieldViewModel = context.viewModel;
                    var fieldAction = fieldViewModel.action;

                    var page = fieldAction.getLink();
                    var criteria = fieldAction.getLinkToPageCriteria();

                    // Add the ID
                    criteria.binding[property_id] = data[property_id];

                    var pageState = fieldViewModel.getLinkToPageInitialState();
                    if (fieldViewModel.linkReplacesPage()) {
                        return fieldViewModel.router.switchToPage(page, pageState, criteria);
                    } else {
                        return fieldViewModel.router.goToPage(page, pageState, criteria);
                    }
                };

                options["createNode"] = function ($node, data) {
                    $node.on("dblclick", activate.bind($node, data));
                };
            }
            
            if (support_wrapping) {
                $divContainer.addClass("vinyl-allow-title-wrapping");    
            }
            
            //
            $divContainer.orgchart(options);
        };

        var setHeader = function(xhr) {
            if (apiKey) {
                //'OxhSS2k5VDgKrpC_UpEMUQ'
                xhr.setRequestHeader('X-API-Key', apiKey);
            }
            
        };
        
        var jsonUrl = context.getParameter("jsonUrl");
        
        $.ajax({
            url: jsonUrl,
            type: 'GET',
            dataType: 'json',
            success: callback,
            error: function(err) { console.log('boo!', err); },
            beforeSend: setHeader
        });

        
    };

    var uninstall = function (holderElement, context) {
        // No implementation
    };

    return {
        callbacks: {
            events: {
                // adding and removing
                install: install,
                uninstall: uninstall
            }
        }
    };
});